"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startApolloServer = exports.createDefaultConfig = void 0;
const apollo_server_express_1 = require("apollo-server-express");
const dotenv_1 = __importDefault(require("dotenv"));
const graphql_tools_1 = require("graphql-tools");
const graphql_upload_1 = require("graphql-upload");
const ip_1 = require("ip");
const resolver_defs_1 = __importDefault(require("./graphql/resolver_defs"));
dotenv_1.default.config();
/**
 * @method createDefaultConfig
 * @param configOptions ConfigOptions
 * @returns ApolloConfig
 */
function createDefaultConfig(configOptions) {
    const config = {};
    config.typeDefs = configOptions.typeDefs;
    config.resolvers = configOptions.resolvers;
    config.context = configOptions.context;
    config.uploads = false;
    config.tracing = process.env.NODE_ENV !== "production";
    config.formatResponse = configOptions.formatResponse;
    config.plugins = [
        {
            requestDidStart(requestContext) {
                return {
                    executionDidStart(executionRequestContext) {
                        return {
                            willResolveField(fieldResolverParams) {
                                configOptions.handleResolver(fieldResolverParams);
                            },
                        };
                    },
                };
            },
        },
    ];
    return config;
}
exports.createDefaultConfig = createDefaultConfig;
/**
 * @method startApolloServer Start apollo server with apply middleware express
 * @param config ApolloConfig
 * @param app Express
 * @param httpServer Server
 * @param host string
 * @param port number
 * @param path string
 * @param uploadOptions UploadOptions | undefined
 * @returns Promise<ApolloServer>
 */
function startApolloServer(config, app, httpServer, host = "0.0.0.0", port = 5000, path = "/graphql", uploadOptions) {
    return __awaiter(this, void 0, void 0, function* () {
        // Init apollo server instance
        if (config.typeDefs && config.resolvers) {
            // const typeArray: (string | DocumentNode)[] = [config.typeDefs, typeDefs];
            // const resolverArray = [resolvers, config.resolvers];
            // config.typeDefs = mergeTypeDefs(typeArray);
            // config.resolvers = mergeResolvers(resolverArray);
            config.schema = graphql_tools_1.mergeSchemas({ schemas: [], typeDefs: [resolver_defs_1.default, config.typeDefs], resolvers: [resolver_defs_1.default, config.resolvers], throwOnConflict: true });
            console.log(config);
            console.log("******************************");
        }
        const apolloServer = new apollo_server_express_1.ApolloServer(config);
        // Start apollo server
        yield apolloServer.start();
        // Setup middleware upload file
        app.use(path, graphql_upload_1.graphqlUploadExpress(uploadOptions));
        // Apply middleware
        apolloServer.applyMiddleware({ app: app, path: path });
        yield new Promise(() => {
            // Server is listening clients
            httpServer.listen({ host: host, port: port }, () => {
                let announcement = {
                    server: httpServer.address(),
                    address: ip_1.address(),
                    message: "GraphQL Server is running!",
                };
                console.log(announcement);
            });
        });
        return apolloServer;
    });
}
exports.startApolloServer = startApolloServer;
